@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -28,12 +28,12 @@
 #include "bond.h"
 #include "molecule.h"
 #include "settings.h"
+#include "theme.h"
 #include "view.h"
 #include "document.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
-#include "libgcpcanvas/gcp-canvas-rich-text.h"
 #include "libgcpcanvas/gcp-canvas-pango.h"
 #include <math.h>
 #include <gcu/element.h>
@@ -43,8 +43,6 @@ using namespace gcu;
 
 #define ATOM_EPSILON 0.1
 
-extern GtkTextTagTable* TextTagTable;
-
 gcpAtom::gcpAtom(): Atom()
 {
 	m_Valence = -1; //unspecified
@@ -131,7 +129,7 @@ void gcpAtom::SetZ(int Z)
 {
 	Atom::SetZ(Z);
 	m_Element = Element::GetElement (m_Z);
-	if (m_Valence = m_Element->GetDefaultValence ())
+	if ((m_Valence = m_Element->GetDefaultValence ()))
 		m_HPos = GetBestSide();
 	else
 		m_nH = 0;
@@ -300,6 +298,7 @@ void gcpAtom::Add (GtkWidget* w)
 		m_Changed--;
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	gcpView* pView = pData->View;
+	gcpTheme *pTheme = pView->GetDoc ()->GetTheme ();
 	if (m_Layout == NULL) {
 		PangoContext* pc = pView->GetPangoContext();
 		m_Layout = pango_layout_new (pc);
@@ -316,10 +315,10 @@ void gcpAtom::Add (GtkWidget* w)
 		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 	}
 	double x, y, xc = 0., yc;
-	m_width =  m_height = 2.0 * pData->Padding;
+	m_width =  m_height = 2.0 * pTheme->GetPadding ();
 	GetCoords (&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
 	GnomeCanvasItem *item;
 	GnomeCanvasGroup *group, *chgp;
 	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
@@ -382,10 +381,10 @@ void gcpAtom::Add (GtkWidget* w)
 		item = gnome_canvas_item_new (
 							group,
 							gnome_canvas_rect_ext_get_type (),
-							"x1", x - m_lbearing - pData->Padding,
-							"y1", y  - m_ascent + m_CHeight - pData->Padding,
-							"x2", x - m_lbearing + m_length + pData->Padding,
-							"y2", y  - m_ascent + m_CHeight + m_height + pData->Padding,
+							"x1", x - m_lbearing - pTheme->GetPadding (),
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
 							"fill_color", (pData->IsSelected (this))? SelectColor: "white",
 							NULL);
 		g_object_set_data (G_OBJECT (group), "rect", item);
@@ -419,7 +418,7 @@ void gcpAtom::Add (GtkWidget* w)
 		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
 		g_object_set_data(G_OBJECT(item), "object", this);
 		if (m_DrawCircle) {
-			double dx = pData->StereoBondWidth / 2.;
+			double dx = pTheme->GetStereoBondWidth () / 2.;
 			item = gnome_canvas_item_new(
 									group,
 									gnome_canvas_ellipse_ext_get_type(),
@@ -435,8 +434,8 @@ void gcpAtom::Add (GtkWidget* w)
 		}
 	}
 	pData->Items[this] = group;
-	m_width /= pData->ZoomFactor;
-	m_height /= pData->ZoomFactor;
+	m_width /= pTheme->GetZoomFactor ();
+	m_height /= pTheme->GetZoomFactor ();
 	/* add charge */
 	int charge = GetCharge ();
 	if (charge) {
@@ -446,8 +445,8 @@ void gcpAtom::Add (GtkWidget* w)
 			x = m_x + m_ChargeDist * cos (m_ChargeAngle);
 			y = m_y - m_ChargeDist * sin (m_ChargeAngle);
 		}
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 		char *fig = NULL;
 		if 	(abs (charge) > 1) {
 			fig = g_strdup_printf ("%d", abs (charge));
@@ -460,33 +459,33 @@ void gcpAtom::Add (GtkWidget* w)
 			pango_layout_set_text (m_ChargeLayout, fig, -1);
 			pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 			m_ChargeWidth = rect.width / PANGO_SCALE;
-			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 		} else {
 			m_ChargeWidth = 0.;
-			m_ChargeTWidth = ChargeSignSize;
+			m_ChargeTWidth = pTheme->GetChargeSignSize ();
 		}
 		switch (align) {
 		case -2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y += ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+			y += pTheme->GetChargeSignHeight () / 2.;
 			break;
 		case -1:
-			xc = x - ChargeSignSize -1.;
+			xc = x - pTheme->GetChargeSignSize () -1.;
 			break;
 		case 0:
 		case -3:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 			break;
 		case 1:
 			xc = x + m_ChargeWidth + 1.;
 			break;
 		case 2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y -= ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+			y -= pTheme->GetChargeSignHeight () / 2.;
 			break;
 		}
 		x = xc - 1.;
-		yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+		yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -510,8 +509,8 @@ void gcpAtom::Add (GtkWidget* w)
 					gnome_canvas_ellipse_ext_get_type (),
 					"x1", xc,
 					"y1", yc,
-					"x2", xc + ChargeSignSize,
-					"y2", yc + ChargeSignSize,
+					"x2", xc + pTheme->GetChargeSignSize (),
+					"y2", yc + pTheme->GetChargeSignSize (),
 					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 					"width_units", 0.5,
 					NULL
@@ -521,14 +520,14 @@ void gcpAtom::Add (GtkWidget* w)
 		path[0].code = ART_MOVETO_OPEN;
 		path[0].x3 = xc + 1.;
 		path[1].code = ART_LINETO;
-		path[1].x3 = xc + ChargeSignSize - 1.;
-		path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+		path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+		path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 		if (charge > 0) {
 			path[2].code = ART_MOVETO_OPEN;
 			path[2].y3 = yc + 1.;
 			path[3].code = ART_LINETO;
-			path[3].y3 = yc + ChargeSignSize - 1.;
-			path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+			path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+			path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 			path[4].code = ART_END;
 		} else
 			path[2].code = ART_END;
@@ -556,10 +555,11 @@ void gcpAtom::Update(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	double x, y, xc = 0., yc;
 	GetCoords(&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
 	GnomeCanvasGroup *group = pData->Items[this];
 	if (m_Changed) {
 		BuildItems (pData);
@@ -570,10 +570,10 @@ void gcpAtom::Update(GtkWidget* w)
 								"y", y - m_ascent + m_CHeight,
 								NULL);
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
-								"x1", x - m_lbearing - pData->Padding,
-								"y1", y  - m_ascent + m_CHeight - pData->Padding,
-								"x2", x - m_lbearing + m_length + pData->Padding,
-								"y2", y  - m_ascent + m_CHeight + m_text_height + pData->Padding,
+								"x1", x - m_lbearing - pTheme->GetPadding (),
+								"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+								"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+								"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding (),
 								NULL);
 		} else {
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
@@ -583,7 +583,7 @@ void gcpAtom::Update(GtkWidget* w)
 									"y2", y + 3,
 								NULL);
 			if (m_DrawCircle) {
-				double dx = pData->StereoBondWidth / 2.;
+				double dx = pTheme->GetStereoBondWidth () / 2.;
 				g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "bullet")),
 										"x1", x - dx,
 										"y1", y - dx,
@@ -603,8 +603,8 @@ void gcpAtom::Update(GtkWidget* w)
 				x = m_x + m_ChargeDist * cos (m_ChargeAngle);
 				y = m_y - m_ChargeDist * sin (m_ChargeAngle);
 			}
-			x *= pData->ZoomFactor;
-			y *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
+			y *= pTheme->GetZoomFactor ();
 			GnomeCanvasItem *figure = (GnomeCanvasItem*) g_object_get_data (G_OBJECT (group), "figure");
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
@@ -620,7 +620,7 @@ void gcpAtom::Update(GtkWidget* w)
 				m_ChargeWidth = rect.width / PANGO_SCALE;
 			} else
 				m_ChargeWidth = 0.;
-			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 			if (figure == NULL && fig != NULL) {
 				figure = gnome_canvas_item_new(
 							GNOME_CANVAS_GROUP (item),
@@ -634,26 +634,26 @@ void gcpAtom::Update(GtkWidget* w)
 			}
 			switch (align) {
 			case -2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y += ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y += pTheme->GetChargeSignHeight () / 2.;
 				break;
 			case -1:
-				xc = x - ChargeSignSize -1.;
+				xc = x - pTheme->GetChargeSignSize () -1.;
 				break;
 			case 0:
 			case -3:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
 				xc = x + m_ChargeWidth + 1.;
 				break;
 			case 2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y -= ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y -= pTheme->GetChargeSignHeight () / 2.;
 				break;
 			}
 			x = xc - 1.;
-			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 			if (fig) {
 				g_object_set (G_OBJECT (figure),
 							"layout", m_ChargeLayout,
@@ -666,22 +666,22 @@ void gcpAtom::Update(GtkWidget* w)
 			g_object_set (G_OBJECT (item),
 						"x1", xc,
 						"y1", yc,
-						"x2", xc + ChargeSignSize,
-						"y2", yc + ChargeSignSize,
+						"x2", xc + pTheme->GetChargeSignSize (),
+						"y2", yc + pTheme->GetChargeSignSize (),
 						NULL);
 			item = g_object_get_data (G_OBJECT (group), "sign");
 			ArtBpath *path = art_new (ArtBpath, 5);
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = xc + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = xc + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = yc + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = yc + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -699,8 +699,8 @@ void gcpAtom::Update(GtkWidget* w)
 				x = (m_x + m_ChargeDist * cos (m_ChargeAngle));
 				y = (m_y - m_ChargeDist * sin (m_ChargeAngle));
 			}
-			x *= pData->ZoomFactor;
-			y *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
+			y *= pTheme->GetZoomFactor ();
 
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
@@ -714,33 +714,33 @@ void gcpAtom::Update(GtkWidget* w)
 				PangoRectangle rect;
 				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				m_ChargeTWidth = m_ChargeWidth + 2 + ChargeSignSize;
+				m_ChargeTWidth = m_ChargeWidth + 2 + pTheme->GetChargeSignSize ();
 			} else {
 				m_ChargeWidth = 0.;
-				m_ChargeTWidth = ChargeSignSize;
+				m_ChargeTWidth = pTheme->GetChargeSignSize ();
 			}
 			switch (align) {
 			case -2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y += ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y += pTheme->GetChargeSignHeight () / 2.;
 				break;
 			case -1:
-				xc = x - ChargeSignSize -1.;
+				xc = x - pTheme->GetChargeSignSize () -1.;
 				break;
 			case 0:
 			case -3:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
 				xc = x + m_ChargeWidth + 1.;
 				break;
 			case 2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y -= ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y -= pTheme->GetChargeSignHeight () / 2.;
 				break;
 			}
 			x = xc - 1.;
-			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 						group,
 						gnome_canvas_group_ext_get_type(),
@@ -764,8 +764,8 @@ void gcpAtom::Update(GtkWidget* w)
 						gnome_canvas_ellipse_ext_get_type (),
 						"x1", xc,
 						"y1", yc,
-						"x2", xc + ChargeSignSize,
-						"y2", yc + ChargeSignSize,
+						"x2", xc + pTheme->GetChargeSignSize (),
+						"y2", yc + pTheme->GetChargeSignSize (),
 						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 						"width_units", .5,
 						NULL
@@ -775,14 +775,14 @@ void gcpAtom::Update(GtkWidget* w)
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = xc + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = xc + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = yc + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = yc + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -1135,13 +1135,13 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	if (item = g_object_get_data(G_OBJECT(group), "bullet"))
+	if ((item = g_object_get_data (G_OBJECT (group), "bullet")))
 		g_object_set (item, "fill_color", chargecolor, NULL);
-	if (item = g_object_get_data(G_OBJECT(group), "figure"))
+	if ((item = g_object_get_data (G_OBJECT (group), "figure")))
 		g_object_set (item, "fill_color", chargecolor, NULL);
-	if (item = g_object_get_data(G_OBJECT(group), "circle"))
+	if ((item = g_object_get_data (G_OBJECT (group), "circle")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
-	if (item = g_object_get_data(G_OBJECT(group), "sign"))
+	if ((item = g_object_get_data (G_OBJECT (group), "sign")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 	Object::SetSelected (w, state);
 }
@@ -1175,11 +1175,12 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 	GnomeCanvasGroup* group = pData->Items[this];
 	void* item;
 	gcpView* pView = pData->View;
+	gcpTheme *pTheme = pView->GetDoc ()->GetTheme ();
 	double x, y;
-	m_width =  m_height = 2.0 * pData->Padding;
+	m_width =  m_height = 2.0 * pTheme->GetPadding ();
 	GetCoords (&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
 	if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
 		int sw, sp;
 		const gchar* symbol = GetSymbol (), *text;
@@ -1236,10 +1237,10 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
-							"x1", x - m_lbearing - pData->Padding,
-							"y1", y  - m_ascent + m_CHeight - pData->Padding,
-							"x2", x - m_lbearing + m_length + pData->Padding,
-							"y2", y  - m_ascent + m_CHeight + m_height + pData->Padding,
+							"x1", x - m_lbearing - pTheme->GetPadding (),
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
 							NULL);
 		
 		item = g_object_get_data (G_OBJECT (group), "symbol");
@@ -1282,7 +1283,7 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 		item = g_object_get_data (G_OBJECT (group), "bullet");
 		if (m_DrawCircle) {
 			if (!item) {
-				double dx = pData->StereoBondWidth / 2.;
+				double dx = pTheme->GetStereoBondWidth () / 2.;
 				item = gnome_canvas_item_new(
 										group,
 										gnome_canvas_ellipse_ext_get_type (),
@@ -1303,8 +1304,8 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 		m_length = m_text_height = 0;
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (group));
 	}
-	m_width /= pData->ZoomFactor;
-	m_height /= pData->ZoomFactor;
+	m_width /= pTheme->GetZoomFactor ();
+	m_height /= pTheme->GetZoomFactor ();
 	if (m_Changed > 0)
 		m_Changed--;
 }
