@@ -31,8 +31,11 @@
 #include "tool.h"
 #include "text.h"
 #include "theme.h"
+#include "window.h"
 #include "libgcpcanvas/gprintable.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
+#include <libgnomevfs/gnome-vfs-file-info.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
 #include <math.h>
 #include <unistd.h>
 #include <pango/pango-context.h>
@@ -41,8 +44,6 @@
 #include <locale.h>
 #include <fstream>
 
-extern bool new_ui;
-
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
 */
@@ -168,37 +169,30 @@ using namespace std;
 gcpView::gcpView (gcpDocument *pDoc, bool Embedded)
 {
 	m_pDoc = pDoc;
+	gcpTheme *pTheme = pDoc->GetTheme ();
 	m_PangoFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoFontDesc, pDoc->GetFontFamily ());
-	pango_font_description_set_style (m_PangoFontDesc, pDoc->GetFontStyle ());
-	pango_font_description_set_weight (m_PangoFontDesc, pDoc->GetFontWeight ());
-	pango_font_description_set_variant (m_PangoFontDesc, pDoc->GetFontVariant ());
-	pango_font_description_set_stretch (m_PangoFontDesc, pDoc->GetFontStretch ());
-	pango_font_description_set_size (m_PangoFontDesc, pDoc->GetFontSize ());
+	pango_font_description_set_family (m_PangoFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoFontDesc, pTheme->GetFontSize ());
 	m_sFontName = pango_font_description_to_string (m_PangoFontDesc);
 	m_PangoSmallFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoSmallFontDesc, pDoc->GetFontFamily ());
-	pango_font_description_set_style (m_PangoSmallFontDesc, pDoc->GetFontStyle ());
-	pango_font_description_set_weight (m_PangoSmallFontDesc, pDoc->GetFontWeight ());
-	pango_font_description_set_variant (m_PangoSmallFontDesc, pDoc->GetFontVariant ());
-	pango_font_description_set_stretch (m_PangoSmallFontDesc, pDoc->GetFontStretch ());
-	pango_font_description_set_size (m_PangoSmallFontDesc, pDoc->GetFontSize () * 2 / 3);
+	pango_font_description_set_family (m_PangoSmallFontDesc, pTheme->GetFontFamily ());
+	pango_font_description_set_style (m_PangoSmallFontDesc, pTheme->GetFontStyle ());
+	pango_font_description_set_weight (m_PangoSmallFontDesc, pTheme->GetFontWeight ());
+	pango_font_description_set_variant (m_PangoSmallFontDesc, pTheme->GetFontVariant ());
+	pango_font_description_set_stretch (m_PangoSmallFontDesc, pTheme->GetFontStretch ());
+	pango_font_description_set_size (m_PangoSmallFontDesc, pTheme->GetFontSize () * 2 / 3);
 	m_sSmallFontName = pango_font_description_to_string (m_PangoSmallFontDesc);
-	m_PangoTinyFontDesc = pango_font_description_new ();
-	pango_font_description_set_family (m_PangoTinyFontDesc, pDoc->GetFontFamily ());
-	pango_font_description_set_style (m_PangoTinyFontDesc, pDoc->GetFontStyle ());
-	pango_font_description_set_weight (m_PangoTinyFontDesc, pDoc->GetFontWeight ());
-	pango_font_description_set_variant (m_PangoTinyFontDesc, pDoc->GetFontVariant ());
-	pango_font_description_set_stretch (m_PangoTinyFontDesc, pDoc->GetFontStretch ());
-	pango_font_description_set_size (m_PangoTinyFontDesc, pDoc->GetFontSize () / 2);
-	m_sTinyFontName = pango_font_description_to_string (m_PangoTinyFontDesc);
 	m_width = 400;
 	m_height = 300;
 	m_ActiveRichText = NULL;
-	if (!(m_bEmbedded = Embedded) && !new_ui)
-		CreateNewWidget ();
+	m_bEmbedded = Embedded;
 	m_UIManager = gtk_ui_manager_new ();
 	m_Dragging = false;
+	m_pWidget = NULL;
 }
 
 gcpView::~gcpView ()
@@ -210,7 +204,6 @@ gcpView::~gcpView ()
 		g_free (m_sSmallFontName);
 	pango_font_description_free (m_PangoFontDesc);
 	pango_font_description_free (m_PangoSmallFontDesc);
-	pango_font_description_free (m_PangoTinyFontDesc);
 	g_object_unref (m_UIManager);
 }
 
@@ -373,21 +366,6 @@ GtkWidget* gcpView::CreateNewWidget()
 		{
 			m_PangoContext = gtk_widget_create_pango_context(m_pWidget);
 			g_object_ref(G_OBJECT(m_PangoContext));
-			// bad hack to get charge sign size.
-/*			if (DefaultChargeSignSize == 0.) {
-				PangoLayout *pl = pango_layout_new (m_PangoContext);
-				char *markup = g_strdup_printf ("<span font_desc=\"%s\">\xE2\x8a\x95</span>", m_sSmallFontName);
-				pango_layout_set_markup (pl, markup, -1);
-				g_free (markup);
-				PangoLayoutIter* iter = pango_layout_get_iter (pl);
-				PangoRectangle ink, log;
-				pango_layout_iter_get_run_extents (iter, &ink, &log);
-				DefaultChargeYAlign = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
-				pango_layout_iter_free (iter);
-				g_object_unref (pl);
-				DefaultChargeSignSize = ink.width / PANGO_SCALE;
-				DefaultChargeSignHeight = log.height / PANGO_SCALE;
-			}*/
 			UpdateFont();
 		}
 	}
@@ -544,15 +522,20 @@ void gcpView::OnDeleteSelection (GtkWidget* w)
 		}
 	}
 	m_pDoc->FinishOperation ();
-	pApp->ActivateMenu ("Copy", false);
-	pApp->ActivateMenu ("Cut", false);
-	pApp->ActivateMenu ("Erase", false);
+	gcpWindow *Win = m_pDoc->GetWindow ();
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 }
 
 GtkTargetEntry const targets[] = {
 	{(char *) GCHEMPAINT_ATOM_NAME,  0, 0},
-	{(char *)"UTF8_STRING", 0, 1},
-	{(char *)"STRING", 0, 2}
+	{(char *) "image/svg",  0, 1},
+	{(char *) "image/svg+xml",  0, 2},
+	{(char *) "image/png",  0, 3},
+	{(char *) "image/jpeg",  0, 4},
+	{(char *) "UTF8_STRING", 0, 1},
+	{(char *) "STRING", 0, 2}
 };
 
 void gcpView::OnCopySelection(GtkWidget* w, GtkClipboard* clipboard)
@@ -582,47 +565,34 @@ void gcpView::OnReceive(GtkClipboard* clipboard, GtkSelectionData* selection_dat
 	if (!pActiveTool || (pActiveTool != pApp->GetTool("Select"))) return;
 	xmlDocPtr xml;
 	m_pData->UnselectAll();
-	switch (*DataType)
-	{
+	switch (*DataType) {
 		case 0:
-			xml = xmlParseMemory((const char*)selection_data->data, selection_data->length);
-			m_pDoc->AddData(xml->children->children);
-			xmlFreeDoc(xml);
+			xml = xmlParseMemory ((const char*) selection_data->data, selection_data->length);
+			m_pDoc->AddData (xml->children->children);
+			xmlFreeDoc (xml);
 			break;
-		case 1:
-			{
-				gcpText* text = new gcpText();
-				GtkTextBuffer *buf = (GtkTextBuffer*)text->GetTextBuffer();
-				gtk_text_buffer_set_text(buf, (const char*)selection_data->data, selection_data->length);
-				GtkTextIter start, end;
-				gtk_text_buffer_get_bounds(buf, &start, &end);
-				gtk_text_buffer_remove_all_tags(buf, &start, &end);
-				gtk_text_buffer_apply_tag_by_name(buf, pApp->GetFontName(), &start, &end);
-				text->OnChanged(buf);
-				m_pDoc->AddObject(text);
-				m_pData->SetSelected(text);
+		case 5: {
+				gcpText* text = new gcpText ();
+				PangoLayout *layout = text->GetLayout ();
+				pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+				text->OnChanged (true);
+				m_pDoc->AddObject (text);
+				m_pData->SetSelected (text);
 			}
 			break;
-		case 2:
-			{
-				gcpText* text = new gcpText();
-				GtkTextBuffer *buf = (GtkTextBuffer*)text->GetTextBuffer();
-				if (!g_utf8_validate( (const char*)selection_data->data, selection_data->length, NULL))
-				{
+		case 6: {
+				gcpText* text = new gcpText ();
+				PangoLayout *layout = text->GetLayout ();
+				if (!g_utf8_validate ((const char*) selection_data->data, selection_data->length, NULL)) {
 					gsize r, w;
-					gchar* newstr = g_locale_to_utf8((const char*)selection_data->data, selection_data->length, &r, &w, NULL);
-					gtk_text_buffer_set_text(buf, newstr, w);
-					g_free(newstr);
-				}
-				else
-					gtk_text_buffer_set_text(buf, (const char*)selection_data->data, selection_data->length);
-				GtkTextIter start, end;
-				gtk_text_buffer_get_bounds(buf, &start, &end);
-				gtk_text_buffer_remove_all_tags(buf, &start, &end);
-				gtk_text_buffer_apply_tag_by_name(buf, pApp->GetFontName(), &start, &end);
-				text->OnChanged(buf);
-				m_pDoc->AddObject(text);
-				m_pData->SetSelected(text);
+					gchar* newstr = g_locale_to_utf8 ((const char*) selection_data->data, selection_data->length, &r, &w, NULL);
+					pango_layout_set_text (layout, newstr, w);
+					g_free (newstr);
+				} else
+					pango_layout_set_text (layout, (const char*) selection_data->data, selection_data->length);
+				text->OnChanged (true);
+				m_pDoc->AddObject (text);
+				m_pData->SetSelected (text);
 			}
 			break;
 	}
@@ -687,9 +657,10 @@ void gcpView::OnCutSelection(GtkWidget* w, GtkClipboard* clipboard)
 		OnCopySelection(w, clipboard);
 		OnDeleteSelection(w);
 	}
-	pApp->ActivateMenu("Copy", false);
-	pApp->ActivateMenu("Cut", false);
-	pApp->ActivateMenu("Erase", false);
+	gcpWindow *Win = m_pDoc->GetWindow ();
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Copy", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Cut", false);
+	Win->ActivateActionWidget ("/MainMenu/EditMenu/Erase", false);
 }
 
 bool gcpView::OnKeyPress(GtkWidget* w, GdkEventKey* event)
@@ -775,7 +746,7 @@ void gcpView::UpdateSize(double x1, double y1, double x2, double y2)
 	}
 }
 
-void gcpView::SetGnomeCanvasRichTextActive(GnomeCanvasRichTextExt* item)
+void gcpView::SetGnomeCanvasPangoActive (GnomeCanvasPango* item)
 {
 	m_ActiveRichText = item;
 	m_Dragging = false;
@@ -798,7 +769,7 @@ void gcpView::OnSelectAll()
 		pActiveTool->AddSelection(m_pData);
 }
 
-void gcpView::ExportImage (const char* filename, const char* type)
+void gcpView::ExportImage (string const &filename, const char* type, int resolution)
 {
 	ArtDRect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
@@ -839,10 +810,10 @@ void gcpView::ExportImage (const char* filename, const char* type)
 		gnome_print_config_unref(config);
 		char buf[256];
 		ifstream *fin = new ifstream (tmpname);
-		ofstream *fout = new ofstream (filename);
-		*fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
-		*fout << "%%BoundingBox: " << (int) (rect.x0 * .75 + ml) << " " << (int) (hp - mt - rect.y1 * .75)  <<  " "  << (int) (rect.x1 * .75 + ml) << " " << (int) (hp - mt - rect.y0 * .75) << endl;
-		*fout << "%%HiResBoundingBox: " << rect.x0 * .75 + ml << " " << hp - mt - rect.y1 * .75 <<  " "  << rect.x1 * .75 + ml << " " << hp - mt - rect.y0 * .75 << endl;
+		ostringstream fout (filename.c_str ());
+		fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
+		fout << "%%BoundingBox: " << (int) (rect.x0 * .75 + ml) << " " << (int) (hp - mt - rect.y1 * .75)  <<  " "  << (int) (rect.x1 * .75 + ml) << " " << (int) (hp - mt - rect.y0 * .75) << endl;
+		fout << "%%HiResBoundingBox: " << rect.x0 * .75 + ml << " " << hp - mt - rect.y1 * .75 <<  " "  << rect.x1 * .75 + ml << " " << hp - mt - rect.y0 * .75 << endl;
 		fin->getline (buf, 256);
 		while (!fin->eof ()) {
 			fin->getline (buf, 256);
@@ -878,8 +849,8 @@ void gcpView::ExportImage (const char* filename, const char* type)
 				else if (!strncmp (buf + 2, "PageResources", strlen ("PageResources")))
 					continue;
 				else if (!strncmp (buf + 2, "EndComments", strlen ("EndComments"))) {
-					 *fout << buf << endl;
-					if (!strcmp (filename + strlen (filename) - 5, ".epsi")) {
+					 fout << buf << endl;
+					if (!filename.compare (filename.length () - 5, 5, ".epsi")) {
 						gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
 						GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
 						GnomeCanvasBuf cbuf;
@@ -895,156 +866,177 @@ void gcpView::ExportImage (const char* filename, const char* type)
 						(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &cbuf);
 						/* use 8 bits depth, no more than 250 bytes per line*/
 						lines = (w + 249) / 250 * h;
-						*fout << "%%BeginPreview: " << w << " " << h << " " << 8 << " " << lines << endl;
-						*fout << hex;
+						fout << "%%BeginPreview: " << w << " " << h << " " << 8 << " " << lines << endl;
+						fout << hex;
 						for (j = 0; j < h; j++) {
 							bytes = 0;
 							for (i = 0; i < 3 * w; i+= 3) {
 								if (bytes == 0)
-									*fout << "%";
+									fout << "%";
 								mask = 0xff - (cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3;
-								*fout << (mask & 0xf);
+								fout << (mask & 0xf);
 								mask = mask >> 4;
-								*fout << mask;
+								fout << mask;
 								bytes++;
 								if (bytes == 250) {
-									*fout << endl;
+									fout << endl;
 									bytes = 0;
 								}
 							}
 							if (bytes)
-								*fout << endl;
+								fout << endl;
 							cbuf.buf += cbuf.buf_rowstride;
 						}
 						/* use 1 bit depth, no more than 128 bytes per line (max width 1024) */
 /*						lines = (w + 1023) / 1024 * h;
-						*fout << "%%BeginPreview: " << w << " " << h << " " << 1 << " " << lines << endl;
-						*fout << hex;
+						fout << "%%BeginPreview: " << w << " " << h << " " << 1 << " " << lines << endl;
+						fout << hex;
 						for (j = 0; j < h; j++) {
 							mask = 1; b = 0;
 							bytes = 0;
 							for (i = 0; i < 3 * w; i+= 3) {
 								if (bytes == 0)
-									*fout << "%";
+									fout << "%";
 								if ((cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3 < 0x80)
 									b |= mask;
 								mask <<= 1;
 								if (mask == 16) {
-									*fout << (int) b;
+									fout << (int) b;
 									mask = 1;
 									b = 0;
 									bytes++;
 									if (bytes == 128) {
-										*fout << endl;
+										fout << endl;
 										bytes = 0;
 									}
 								}
 							}
 							if (mask != 1) {
-								*fout << (int) b;
+								fout << (int) b;
 								bytes++;
 							}
 							if (bytes)
-								*fout << endl;
+								fout << endl;
 							cbuf.buf += cbuf.buf_rowstride;
 						}*/
-						*fout << dec;
-						*fout << "%%EndPreview" << endl;
+						fout << dec;
+						fout << "%%EndPreview" << endl;
 						g_object_unref (pixbuf);
 					}
 				} else if (!strncmp (buf + 2, "BeginProlog", strlen ("BeginProlog"))) {
-					 *fout << buf << endl;
-					 *fout << "save" << endl;
-					 *fout << "countdictstack" << endl;
-					 *fout << "mark" << endl;
-					 *fout << "newpath" << endl;
-					 *fout << "/showpage {} def" << endl;
-					 *fout << "/setpagedevice {pop} def" << endl;
-					 *fout << "%%EndProlog" << endl;
-					 *fout << "%%Page 1 1" << endl;
+					 fout << buf << endl;
+					 fout << "save" << endl;
+					 fout << "countdictstack" << endl;
+					 fout << "mark" << endl;
+					 fout << "newpath" << endl;
+					 fout << "/showpage {} def" << endl;
+					 fout << "/setpagedevice {pop} def" << endl;
+					 fout << "%%EndProlog" << endl;
+					 fout << "%%Page 1 1" << endl;
 				} else if (!strncmp (buf + 2, "Trailer", strlen ("Trailer"))) {
-					 *fout << buf << endl;
-					 *fout << "cleartomark" << endl;
-					 *fout << "countdictstack" << endl;
-					 *fout << "exch sub { end } repeat" << endl;
-					 *fout << "restore" << endl;
+					 fout << buf << endl;
+					 fout << "cleartomark" << endl;
+					 fout << "countdictstack" << endl;
+					 fout << "exch sub { end } repeat" << endl;
+					 fout << "restore" << endl;
 				} else 
-					 *fout << buf << endl;
+					 fout << buf << endl;
 			} else
-			 *fout << buf << endl;
+			 fout << buf << endl;
 		}
 		fin->close ();
 		delete fin;
-		fout->close ();
-		delete fout;
+		GnomeVFSHandle *handle = NULL;
+		GnomeVFSFileSize n;
+		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK)
+			gnome_vfs_write (handle, fout.str ().c_str (), (GnomeVFSFileSize) fout.str ().size (), &n);
 	} else if (!strcmp (type, "svg")) {
-		xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
-		char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
-		char *buf;
-		setlocale (LC_NUMERIC, "C");
-		xmlNewDtd (doc,
-			   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
-			   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
-		xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
-		xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
-		xmlSetNs (doc->children, ns);
-		xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
-		rect.x0 = floor (rect.x0);
-		rect.y0 = floor (rect.y0);
-		rect.x1 = ceil (rect.x1);
-		rect.y1 = ceil (rect.y1);
-		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-		xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
-		g_free (buf);
-		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-		xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
-		g_free (buf);
-		xmlNodePtr node;
-		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
-		xmlAddChild (doc->children, node);
-		buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
-		xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
-		g_free (buf);
-		buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
-		xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
-		g_free (buf);
-		xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
-		xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
-		if (rect.x0 != 0. || rect.y0 != 0) {
-			node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
-			xmlAddChild (doc->children, node);
-			buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
-			xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
-			g_free (buf);
-		} else
-			node = doc->children;
-		g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
-		setlocale (LC_NUMERIC, old_num_locale);
-		g_free (old_num_locale);
+		xmlDocPtr doc = BuildSVG ();
 		xmlIndentTreeOutput = true;
 		xmlKeepBlanksDefault (0);
-		xmlSaveFormatFile (filename, doc, true);
+		xmlSaveFormatFile (filename.c_str (), doc, true);
 		xmlFreeDoc (doc);
 	} else {
+		double zoom;
+		if (resolution > 0) {
+			int screenres = m_pDoc->GetApp ()->GetScreenResolution ();
+			zoom = (double) resolution / screenres;
+			w = (int) rint ((double) w * zoom);
+			h = (int) rint ((double) h * zoom);
+		} else
+		zoom = 1.;
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
 		gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
 		GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
 		gdk_pixbuf_fill (pixbuf, 0xffffffff);
 		GnomeCanvasBuf buf;
 		buf.buf = gdk_pixbuf_get_pixels (pixbuf);
-		buf.rect.x0 = (int) floor (rect.x0);
-		buf.rect.x1 = (int) ceil (rect.x1);
-		buf.rect.y0 = (int) floor (rect.y0);
-		buf.rect.y1 = (int) ceil (rect.y1);
+		buf.rect.x0 = (int) floor (rect.x0 * zoom);
+		buf.rect.x1 = (int) ceil (rect.x1 * zoom);
+		buf.rect.y0 = (int) floor (rect.y0 * zoom);
+		buf.rect.y1 = (int) ceil (rect.y1 * zoom);
 		buf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
 		buf.bg_color = 0xffffff;
 		buf.is_buf = 1;
 		(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &buf);
-		gdk_pixbuf_save (pixbuf, filename, type, NULL, NULL);
+		gdk_pixbuf_save (pixbuf, filename.c_str (), type, NULL, NULL);
 		g_object_unref (pixbuf);
+		// restore zoom level
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), m_pData->Zoom);
 	}
 	m_pData->ShowSelection (true);
 }
 
+xmlDocPtr gcpView::BuildSVG ()
+{
+	ArtDRect rect;
+	m_pData->GetObjectBounds (m_pDoc, &rect);
+	xmlDocPtr doc = xmlNewDoc ((const xmlChar*)"1.0");
+	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
+	char *buf;
+	setlocale (LC_NUMERIC, "C");
+	xmlNewDtd (doc,
+		   (const xmlChar*)"svg", (const xmlChar*)"-//W3C//DTD SVG 1.1//EN",
+		   (const xmlChar*)"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
+	xmlDocSetRootElement (doc, xmlNewDocNode (doc, NULL, (const xmlChar*)"svg", NULL));
+	xmlNsPtr ns = xmlNewNs (doc->children, (const xmlChar*)"http://www.w3.org/2000/svg", NULL);
+	xmlSetNs (doc->children, ns);
+	xmlNewProp (doc->children, (const xmlChar*)"version", (const xmlChar*)"1.1");
+	rect.x0 = floor (rect.x0);
+	rect.y0 = floor (rect.y0);
+	rect.x1 = ceil (rect.x1);
+	rect.y1 = ceil (rect.y1);
+	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+	xmlNewProp (doc->children, (const xmlChar*)"width", (const xmlChar*)buf);
+	g_free (buf);
+	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+	xmlNewProp (doc->children, (const xmlChar*)"height", (const xmlChar*)buf);
+	g_free (buf);
+	xmlNodePtr node;
+	node = xmlNewDocNode (doc, NULL, (const xmlChar*)"rect", NULL);
+	xmlAddChild (doc->children, node);
+	buf = g_strdup_printf ("%g", rect.x1 - rect.x0);
+	xmlNewProp (node, (const xmlChar*)"width", (const xmlChar*)buf);
+	g_free (buf);
+	buf = g_strdup_printf ("%g", rect.y1 - rect.y0);
+	xmlNewProp (node, (const xmlChar*)"height", (const xmlChar*)buf);
+	g_free (buf);
+	xmlNewProp (node, (const xmlChar*)"stroke", (const xmlChar*)"none");
+	xmlNewProp (node, (const xmlChar*)"fill", (const xmlChar*)"white");
+	if (rect.x0 != 0. || rect.y0 != 0) {
+		node = xmlNewDocNode (doc, NULL, (const xmlChar*)"g", NULL);
+		xmlAddChild (doc->children, node);
+		buf = g_strdup_printf ("translate(%g,%g)", - rect.x0, - rect.y0);
+		xmlNewProp (node, (const xmlChar*)"transform", (const xmlChar*)buf);
+		g_free (buf);
+	} else
+		node = doc->children;
+	g_printable_export_svg (G_PRINTABLE (m_pData->Group), doc, node);
+	setlocale (LC_NUMERIC, old_num_locale);
+	g_free (old_num_locale);
+	return doc;
+}
+
 void gcpView::EnsureSize ()
 {
 	GnomeCanvas *canvas = GNOME_CANVAS (m_pWidget);
@@ -1061,3 +1053,9 @@ void gcpView::Zoom (double zoom)
 	gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_pWidget), zoom);
 	EnsureSize ();
 }
+
+void gcpView::ShowCursor (bool show)
+{
+	if (m_ActiveRichText)
+		g_object_set (G_OBJECT (m_ActiveRichText), "editing", show, NULL);
+}
