@@ -4,7 +4,7 @@
  * GChemPaint bonds plugin
  * bondtool.cc
  *
- * Copyright (C) 2001-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,11 +26,11 @@
 #include <math.h>
 #include "bondtool.h"
 #include "lib/settings.h"
-#include "lib/mendeleiev.h"
 #include "lib/document.h"
 #include "lib/application.h"
 #include "lib/atom.h"
 #include "lib/bond.h"
+#include "lib/theme.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 
 gcpBondTool::gcpBondTool (gcpApplication *App, string ToolId, unsigned nPoints): gcpTool (App, ToolId)
@@ -94,7 +94,7 @@ bool gcpBondTool::OnClicked ()
 					gcpBond* bond = (gcpBond*) ((Atom*) m_pObject)->GetFirstBond (i);
 					m_dAngle = bond->GetAngle2D ((gcpAtom*) m_pObject);
 					m_dAngle += (m_nState & GDK_LOCK_MASK)?
-						DefaultBondAngle: -DefaultBondAngle;
+						pDoc->GetBondAngle (): -pDoc->GetBondAngle ();
 					break;
 				}
 				case 2: {
@@ -126,8 +126,8 @@ bool gcpBondTool::OnClicked ()
 		points->coords[1] = m_y0;
 	}
 	double a = m_dAngle * M_PI / 180.;
-	m_x1 =  m_x0 + DefaultBondLength * m_dZoomFactor * cos (a);
-	m_y1 =  m_y0 - DefaultBondLength * m_dZoomFactor * sin (a);
+	m_x1 =  m_x0 + pDoc->GetBondLength () * m_dZoomFactor * cos (a);
+	m_y1 =  m_y0 - pDoc->GetBondLength () * m_dZoomFactor * sin (a);
 	GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x1, m_y1);
 	if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 	Object* pObject = NULL;
@@ -164,9 +164,11 @@ bool gcpBondTool::OnClicked ()
 void gcpBondTool::OnDrag()
 {
 	double x1, y1, x2, y2;
+	gcpDocument* pDoc = m_pView->GetDoc ();
+	gcpTheme *Theme = pDoc->GetTheme ();
 	if ((m_pObject) && (m_pObject->GetType() == BondType))
 	{
-		if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (DefaultPadding + DefaultBondWidth / 2) * m_dZoomFactor)
+		if (((gcpBond*)m_pObject)->GetDist(m_x / m_dZoomFactor, m_y / m_dZoomFactor) < (Theme->GetPadding () + Theme->GetBondWidth () / 2) * m_dZoomFactor)
 		{
 			if (!m_bChanged)
 			{
@@ -193,7 +195,11 @@ void gcpBondTool::OnDrag()
 		GnomeCanvasItem* pItem = gnome_canvas_get_item_at(GNOME_CANVAS(m_pWidget), m_x, m_y);
 		if (pItem == (GnomeCanvasItem*)m_pBackground) pItem = NULL;
 		Object* pObject = NULL;
-		if (pItem) pObject = (Object*)g_object_get_data(G_OBJECT(pItem), "object");
+		if (pItem) {
+			pObject = (Object*) g_object_get_data(G_OBJECT(pItem), "object");
+			if (pObject == m_pObject || (pObject->GetType() == FragmentType) && dynamic_cast<gcpFragment*>(pObject)->GetAtom () == m_pObject)
+				return;
+		}
 		double dAngle;
 		m_pAtom = NULL;
 		if (MergeAtoms && pObject)
@@ -253,8 +259,8 @@ void gcpBondTool::OnDrag()
 			}
 			else
 			{
-				m_x1 = m_x0 + DefaultBondLength * m_dZoomFactor * cos(m_dAngle / 180 * M_PI);
-				m_y1 = m_y0 - DefaultBondLength * m_dZoomFactor * sin(m_dAngle / 180 * M_PI);
+				m_x1 = m_x0 + pDoc->GetBondLength () * m_dZoomFactor * cos(m_dAngle / 180 * M_PI);
+				m_y1 = m_y0 - pDoc->GetBondLength () * m_dZoomFactor * sin(m_dAngle / 180 * M_PI);
 			}
 		}
 		char tmp[32];
@@ -401,6 +407,7 @@ void gcpBondTool::OnRelease()
 void gcpBondTool::Draw()
 {
 	double x1, y1, x2, y2;
+	gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	points->coords[2] = m_x1;
 	points->coords[3] = m_y1;
 	m_pItem = gnome_canvas_item_new(
@@ -408,7 +415,7 @@ void gcpBondTool::Draw()
 								gnome_canvas_line_get_type(),
 								"points", points,
 								"fill_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								NULL);
 	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(m_pItem), &x1, &y1, &x2, &y2);
 	gnome_canvas_request_redraw(GNOME_CANVAS(m_pWidget), (int)x1, (int)y1, (int)x2, (int)y2);
@@ -418,6 +425,7 @@ void gcpBondTool::UpdateBond()
 {
 	double x1, y1, x2, y2;
 	int i = 1;
+	gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	BondOrder = ((gcpBond*)m_pObject)->GetOrder();
 	if (((gcpBond*)m_pObject)->GetType() == NormalBondType) ((gcpBond*)m_pObject)->IncOrder();
 	m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
@@ -432,7 +440,7 @@ void gcpBondTool::UpdateBond()
 						gnome_canvas_line_get_type(),
 						"points", points,
 						"fill_color", AddColor,
-						"width_units", m_pData->BondWidth,
+						"width_units", pTheme->GetBondWidth (),
 						NULL);
 	}
 	gnome_canvas_item_get_bounds(GNOME_CANVAS_ITEM(m_pItem), &x1, &y1, &x2, &y2);
@@ -463,14 +471,14 @@ void gcpBondTool::SetType(gcpBond* pBond)
 	pBond->SetType(NormalBondType);
 }
 
-static void on_length_changed (GtkSpinButton *btn)
+static void on_length_changed (GtkSpinButton *btn, gcpBondTool *tool)
 {
-	DefaultBondLength = gtk_spin_button_get_value (btn);
+	tool->SetLength (gtk_spin_button_get_value (btn));
 }
 
-static void on_angle_changed (GtkSpinButton *btn)
+static void on_angle_changed (GtkSpinButton *btn, gcpBondTool *tool)
 {
-	DefaultBondAngle = gtk_spin_button_get_value (btn);
+	tool->SetAngle (gtk_spin_button_get_value (btn));
 }
 
 static void on_merge_toggled (GtkToggleButton *btn)
@@ -478,26 +486,34 @@ static void on_merge_toggled (GtkToggleButton *btn)
 	MergeAtoms = gtk_toggle_button_get_active (btn);
 }
 
+void gcpBondTool::SetAngle (double angle)
+{
+	m_pApp->GetActiveDocument ()->SetBondAngle (angle);
+}
+
+void gcpBondTool::SetLength (double length)
+{
+	m_pApp->GetActiveDocument ()->SetBondLength (length);
+}
+
 GtkWidget *gcpBondTool::GetPropertyPage ()
 {
 	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/bond.glade", "bond", GETTEXT_PACKAGE);
 	m_LengthBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-length"));
-	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), NULL);
+	g_signal_connect (m_LengthBtn, "value-changed", G_CALLBACK (on_length_changed), this);
 	m_AngleBtn = GTK_SPIN_BUTTON (glade_xml_get_widget (xml, "bond-angle"));
-	g_signal_connect (m_AngleBtn, "value-changed", G_CALLBACK (on_angle_changed), NULL);
+	g_signal_connect (m_AngleBtn, "value-changed", G_CALLBACK (on_angle_changed), this);
 	m_MergeBtn = GTK_TOGGLE_BUTTON (glade_xml_get_widget (xml, "merge"));
 	g_signal_connect (m_MergeBtn, "toggled", G_CALLBACK (on_merge_toggled), NULL);
 	return glade_xml_get_widget (xml, "bond");
 }
 
-extern bool new_ui;
 void gcpBondTool::Activate ()
 {
-	if (new_ui) {
-		gtk_spin_button_set_value (m_LengthBtn, DefaultBondLength);
-		gtk_spin_button_set_value (m_AngleBtn, DefaultBondAngle);
-		gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
-	}
+	gcpDocument *pDoc = m_pApp->GetActiveDocument ();
+	gtk_spin_button_set_value (m_LengthBtn, pDoc->GetBondLength ());
+	gtk_spin_button_set_value (m_AngleBtn, pDoc->GetBondAngle ());
+	gtk_toggle_button_set_active (m_MergeBtn, MergeAtoms);
 }
 
 gcpUpBondTool::gcpUpBondTool(gcpApplication *App): gcpBondTool(App, "UpBond", 3)
@@ -511,10 +527,11 @@ gcpUpBondTool::~gcpUpBondTool()
 void gcpUpBondTool::Draw()
 {
 	double dx, dy, x1, y1, x2, y2;
+	gcpTheme *Theme = m_pView->GetDoc ()->GetTheme ();
 	x1 = sqrt(square(m_x1 - m_x0) + square(m_y1 - m_y0));
 	if (x1 == 0) return;
-	dx = (m_y0 - m_y1) / x1 * m_pData->StereoBondWidth / 2;
-	dy = (m_x1 - m_x0) / x1 * m_pData->StereoBondWidth / 2;
+	dx = (m_y0 - m_y1) / x1 * Theme->GetStereoBondWidth () / 2;
+	dy = (m_x1 - m_x0) / x1 * Theme->GetStereoBondWidth () / 2;
 	points->coords[2] = m_x1 + dx;
 	points->coords[3] = m_y1 + dy;
 	points->coords[4] = m_x1 - dx;
@@ -577,32 +594,33 @@ gcpDownBondTool::~gcpDownBondTool()
 void gcpDownBondTool::Draw()
 {
 	double dx, dy, dx1, dy1, length;
+	gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	m_pItem = gnome_canvas_item_new(m_pGroup, gnome_canvas_group_ext_get_type(), NULL);
 	length = sqrt(square(m_x1 - m_x0) + square(m_y1 - m_y0));
 	if (length == 0.0) return;
-	int n = int(floor(length / (m_pData->HashDist + m_pData->HashWidth)));
-	dx1 = (m_x1 - m_x0) / length * m_pData->HashWidth;
-	dy1 = (m_y1 - m_y0) / length * m_pData->HashWidth;
-	dx = (m_y0 - m_y1) / length * m_pData->StereoBondWidth / 2;
-	dy = (m_x1 - m_x0) / length * m_pData->StereoBondWidth / 2;
+	int n = int(floor(length / (pTheme->GetHashDist () + pTheme->GetHashWidth ())));
+	dx1 = (m_x1 - m_x0) / length * pTheme->GetHashWidth ();
+	dy1 = (m_y1 - m_y0) / length * pTheme->GetHashWidth ();
+	dx = (m_y0 - m_y1) / length * pTheme->GetStereoBondWidth () / 2;
+	dy = (m_x1 - m_x0) / length * pTheme->GetStereoBondWidth () / 2;
 	points->coords[0] = m_x0 + dx;
 	points->coords[1] = m_y0 + dy;
 	points->coords[2] = m_x0 - dx;
 	points->coords[3] = m_y0 - dy;
-	dx *= (1 - m_pData->HashWidth / length);
-	dy *= (1 - m_pData->HashWidth / length);
+	dx *= (1 - pTheme->GetHashWidth () / length);
+	dy *= (1 - pTheme->GetHashWidth () / length);
 	points->coords[4] = m_x0 + dx1 - dx;
 	points->coords[5] = m_y0 + dy1 - dy;
 	points->coords[6] = m_x0 + dx1 + dx;
 	points->coords[7] = m_y0 + dy1 + dy;
-	dx = (m_x1 - m_x0) / length * (m_pData->HashDist + m_pData->HashWidth)
-		- (m_y0 - m_y1) / length * m_pData->StereoBondWidth / 2 * (m_pData->HashDist + m_pData->HashWidth) / length;
-	dy = (m_y1 - m_y0) / length * (m_pData->HashDist + m_pData->HashWidth)
-		- (m_x1 - m_x0) / length * m_pData->StereoBondWidth / 2 *  (m_pData->HashDist + m_pData->HashWidth) / length;
-	dx1 = (m_x1 - m_x0) / length * (m_pData->HashDist + m_pData->HashWidth)
-		+ (m_y0 - m_y1) / length * m_pData->StereoBondWidth / 2 *  (m_pData->HashDist + m_pData->HashWidth) / length;
-	dy1 = (m_y1 - m_y0) / length * (m_pData->HashDist + m_pData->HashWidth)
-		+ (m_x1 - m_x0) / length * m_pData->StereoBondWidth / 2 *  (m_pData->HashDist + m_pData->HashWidth) / length;
+	dx = (m_x1 - m_x0) / length * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+		- (m_y0 - m_y1) / length * pTheme->GetStereoBondWidth () / 2 * (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / length;
+	dy = (m_y1 - m_y0) / length * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+		- (m_x1 - m_x0) / length * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / length;
+	dx1 = (m_x1 - m_x0) / length * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+		+ (m_y0 - m_y1) / length * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / length;
+	dy1 = (m_y1 - m_y0) / length * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+		+ (m_x1 - m_x0) / length * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / length;
 	gnome_canvas_item_new(
 						GNOME_CANVAS_GROUP(m_pItem),
 						gnome_canvas_polygon_get_type(),
@@ -676,10 +694,11 @@ gcpForeBondTool::~gcpForeBondTool()
 void gcpForeBondTool::Draw()
 {
 	double dx, dy, x1, y1, x2, y2;
+	gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	x1 = sqrt(square(m_x1 - m_x0) + square(m_y1 - m_y0));
 	if (x1 == 0) return;
-	dx = (m_y0 - m_y1) / x1 * m_pData->StereoBondWidth / 2;
-	dy = (m_x1 - m_x0) / x1 * m_pData->StereoBondWidth / 2;
+	dx = (m_y0 - m_y1) / x1 * pTheme->GetStereoBondWidth () / 2;
+	dy = (m_x1 - m_x0) / x1 * pTheme->GetStereoBondWidth () / 2;
 	points->coords[0] = m_x0 + dx;
 	points->coords[1] = m_y0 + dy;
 	points->coords[2] = m_x1 + dx;
@@ -732,6 +751,7 @@ gcpSquiggleBondTool::~gcpSquiggleBondTool()
 void gcpSquiggleBondTool::Draw()
 {
 	GnomeCanvasPathDef *path_def;
+	gcpTheme *pTheme = m_pView->GetDoc ()->GetTheme ();
 	path_def = gnome_canvas_path_def_new();
 	gnome_canvas_path_def_moveto(path_def, m_x0, m_y0);
 	double x = m_x0, y = m_y0, dx, dy, length, x1, x2, y1, y2;
@@ -759,7 +779,7 @@ void gcpSquiggleBondTool::Draw()
 								m_pGroup,
 								gnome_canvas_bpath_get_type(),
 								"outline_color", AddColor,
-								"width_units", m_pData->BondWidth,
+								"width_units", pTheme->GetBondWidth (),
 								"bpath", path_def,
 								NULL);
 	gnome_canvas_path_def_unref(path_def);
