@@ -4,7 +4,7 @@
  * GChemPaint bonds plugin
  * bondtool.h 
  *
- * Copyright (C) 2001-2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -35,9 +35,13 @@ public:
 	gcpBondTool(gcpApplication *App, string ToolId = "Bond", unsigned nPoints = 2);
 	virtual ~gcpBondTool();
 	
-	virtual bool OnClicked();
-	virtual void OnDrag();
-	virtual void OnRelease();
+	bool OnClicked();
+	void OnDrag();
+	void OnRelease();
+	GtkWidget *GetPropertyPage ();
+	void Activate ();
+	void SetAngle (double angle);
+	void SetLength (double length);
 
 protected:
 	virtual void Draw();
@@ -53,6 +57,8 @@ protected:
 
 private:
 	gcpOperation *m_pOp;
+	GtkSpinButton *m_LengthBtn, *m_AngleBtn;
+	GtkToggleButton *m_MergeBtn;
 };
 
 class gcpUpBondTool: public gcpBondTool
