@@ -4,9 +4,7 @@
  * GChemPaint text plugin
  * fragmenttool.h 
  *
- * Copyright (C) 2003-2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2003-2004 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -43,6 +41,7 @@ public:
 	virtual bool CutSelection(GtkClipboard *clipboard);
 	virtual bool OnReceive(GtkClipboard *clipboard, GtkSelectionData *data, int type);
 	void OnGetData(GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info);
+	GtkWidget *GetPropertyPage () {return NULL;}
 
 protected:
 	virtual bool Unselect();
