@@ -26,6 +26,8 @@
 #define GCHEMPAINT_TEXT_TOOL_H
 
 #include "lib/tool.h"
+#include <gcu/macros.h>
+#include <goffice/gtk/go-color-selector.h>
 #include <list>
 
 using namespace std;
@@ -41,8 +43,6 @@ public:
 	virtual void Activate ();
 	virtual bool OnEvent (GdkEvent *event);
 	virtual bool NotifyViewChange ();
-	bool OnToggled (GtkWidget *btn);
-	bool OnFont ();
 	virtual bool DeleteSelection ();
 	virtual bool CopySelection (GtkClipboard *clipboard);
 	virtual bool CutSelection (GtkClipboard *clipboard);
@@ -56,6 +56,14 @@ public:
 	void OnSelectFamily (GtkTreeSelection *selection);
 	void OnSelectFace (GtkTreeSelection *selection);
 	void OnSelectSize (int size);
+	void OnSizeChanged ();
+	void SetSizeFull (bool update_list);
+	void UpdateAttributeList ();
+	int GetIndex ();
+	void OnUnderlineChanged (unsigned underline);
+	void OnStriketroughToggled (bool strikethrough);
+	void OnPositionChanged (unsigned position);
+	void OnForeColorChanged (GOColor color);
 
 protected:
 	virtual bool Unselect ();
@@ -65,26 +73,39 @@ private:
 	void SelectBestFontFace ();
 
 protected:
-	GnomeCanvasRichTextExt* m_Active;
+	GnomeCanvasPango* m_Active;
 	list<xmlNodePtr> m_UndoList, m_RedoList;
 	xmlNodePtr m_CurNode, m_InitNode;
 
 private:
 	bool m_bUndo;
+	bool m_Bold, m_Italic, m_Subscript, m_Superscript;
 	PangoFontDescription *m_FontDesc, *m_DefaultFontDesc;
 	GtkListStore *m_FamilyList, *m_FaceList, *m_SizeList;
+	GtkTreeView *m_FamilyTree;
 	GtkTreeView *m_FacesTree;
+	GtkTreeView *m_SizesTree;
 	GtkEntry *m_SizeEntry;
+	GOSelector *m_ColorSelector;
 	map<string, PangoFontFamily*> m_Families;
 	map<string, PangoFontFace*> m_Faces;
-	PangoStyle m_Style;
-	PangoWeight m_Weight;
-	PangoStretch m_Stretch;
-	PangoVariant m_Variant;
-	int m_Size;
-	guint m_FaceSignal, m_SizeSignal;
-	gpointer m_FaceSel, m_SizeSel;
+	guint m_FamilySignal, m_FaceSignal, m_SizeSignal;
+	gpointer m_FamilySel, m_FaceSel, m_SizeSel;
+	GtkComboBox *m_UnderlineBox;
+	GtkToggleButton *m_StrikethroughBtn;
 	bool m_Scalable, m_Dirty;
+	gulong m_SelSignal, m_UnderlineSignal, m_StrikethroughSignal, m_ForeSignal;
+
+GCU_PROP (char const *, FamilyName)
+GCU_PROP (PangoStyle, Style)
+GCU_PROP (PangoWeight, Weight)
+GCU_PROP (PangoStretch, Stretch)
+GCU_PROP (PangoVariant, Variant)
+GCU_PROP (PangoUnderline, Underline)
+GCU_PROP (int, Size)
+GCU_PROP (int, Rise)
+GCU_PROP (bool, Strikethrough)
+GCU_PROP (GOColor, Color);
 };
 
 #endif	//GCHEMPAINT_TEXT_TOOL_H
