@@ -24,9 +24,11 @@
 
 #include "gchempaint-config.h"
 #include "standaloneapp.h"
+#include <goffice/utils/go-file.h>
 #include <gtk/gtk.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <libgnomevfs/gnome-vfs-mime.h>
 
 extern "C" {
 	void gnome_authentication_manager_init ();
@@ -44,10 +46,11 @@ void cb_print_version (const gchar *option_name, const gchar *value, gpointer da
 	exit (0);
 }
 
+
 static GOptionEntry entries[] = 
 {
-  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "prints GChemPaint version", NULL },
-   { NULL }
+  { "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, "Prints GChemPaint version", NULL },
+  { NULL }
 };
 
 int main(int argc, char *argv[])
@@ -74,14 +77,14 @@ int main(int argc, char *argv[])
 			g_error_free (error);
 			return -1;
 		}
-	} else {
-		argc --;
-		argv ++;
 	}
+	argv ++;
+	argc --;
 
 	gcpStandaloneApp* App = new gcpStandaloneApp();
 	GnomeVFSURI *uri, *auri;
 	char *path = g_get_current_dir (), *dir;
+	char const *mime_type;
 	dir = g_strconcat (path, "/", NULL);
 	g_free (path);
 	uri = gnome_vfs_uri_new (dir);
@@ -96,7 +99,10 @@ int main(int argc, char *argv[])
 		}
 		auri = gnome_vfs_uri_resolve_relative (uri, *argv);
 		path = gnome_vfs_uri_to_string (auri, GNOME_VFS_URI_HIDE_NONE);
-		App->FileProcess(path, 0, false, NULL);
+		mime_type = go_get_mime_type (path);
+		if (!mime_type)
+			mime_type = gnome_vfs_get_file_mime_type_fast (path, NULL);
+		App->FileProcess(path, mime_type, false, NULL);
 		g_free (path);
 		gnome_vfs_uri_unref (auri);
 		argv++;
@@ -107,8 +113,6 @@ int main(int argc, char *argv[])
 	if (App->GetDocsNumber () == 0)
 		App->OnFileNew();
 
-	App->Show ();
-
 	gtk_main();
 	delete App;
 
